//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// LessonSeries class - holds a series of lessons
//
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;

/**
 * Holds a series of lessons.
 */
public class LessonSeries {

	// message catalog information
	private static ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );

	// attributes of the lesson series
	private String		series_id;
	private String		descr;
	private Vector		lessons		= new Vector();


	/**
	 * Factory method to return a complete lesson series from the
	 * series name given.
	 */
	public static LessonSeries create_lessonseries( String name ) {

		// try to find the resource bundle for the given name
		ResourceBundle resource = null;
		try {
			resource = ResourceBundle.getBundle(
							"LessonSeries" + name );
		}
		catch ( MissingResourceException e ) {}
		catch ( ClassCastException e ) {}
		if ( resource != null )
			return ( new LessonSeries( resource ));

		// no resource bundle, so try for an old-style instantiation
		// of the class LessonSeriesXXX.class
		try {
			Class class_handle = Class.forName(
							"LessonSeries" + name );
			return ( (LessonSeries)(class_handle.newInstance()) );
		} catch( Exception e ) {
			return ( null );
		}
	}

	/**
	 * Create a new LessonSeries from the given resource bundle.
	 */
	public LessonSeries( ResourceBundle resource ) {

		// find any series id and description, and store them
		try {
			this.series_id = resource.getString( "series_id" );
		}
		catch ( MissingResourceException e ) {
			this.series_id = 
				messages.getString( "LessonSeries.untitled" );
			String[] fmtargs = { series_id };
			System.err.println( MessageFormat.format(
				messages.getString( "LessonSeries.missingid" ),
								fmtargs ));
		}
		try {
			this.descr = resource.getString( "description" );
		}
		catch ( MissingResourceException e ) {
			this.descr =
				messages.getString( "LessonSeries.nodescr" );
			String[] fmtargs = { series_id };
			System.err.println( MessageFormat.format(
				messages.getString(
					"LessonSeries.missingdescription" ),
								fmtargs ));
		}

		// find the index of lessons
		String lesson_index;
		try {
			lesson_index = resource.getString( "lesson_index" );
		}
		catch ( MissingResourceException e ) {
			String[] fmtargs = { series_id };
			System.err.println( MessageFormat.format(
					messages.getString(
						"LessonSeries.missingindex" ),
								fmtargs ));
			return;
		}

		// handle each entry in the resource
		StringTokenizer lessons	= new StringTokenizer( lesson_index );
		while ( lessons.hasMoreTokens() ) {
			String lesson_ref = lessons.nextToken();

			// look for this string lesson type, and if there is
			// no definition then call it a tutorial
			LessonType lesson_type	= LessonType.TUTORIAL;
			String s_lesson_type	= null;
			try {
				s_lesson_type = resource.getString(
							lesson_ref + ".type" );
			}
			catch ( MissingResourceException e ) {
				String[] fmtargs = { series_id, lesson_ref };
				System.err.println(
					MessageFormat.format(
						messages.getString(
						"LessonSeries.missingtype" ),
								fmtargs ));
			}

			// if we got a string, match it to the known types
			if ( s_lesson_type != null )
				lesson_type = LessonType.find_lessontype(
						s_lesson_type.toUpperCase() );
			if ( lesson_type == null ) {

				// invalid lesson type encountered
				String[] fmtargs = { series_id, lesson_ref };
				System.err.println(
					MessageFormat.format(
						messages.getString(
						"LessonSeries.badtype" ),
								fmtargs ));

				// again, default to tutorial
				lesson_type = LessonType.TUTORIAL;
			}

			// create the new lesson
			if ( lesson_type == LessonType.DRILL )
				this.append(
					new DrillLesson( series_id,resource,
								lesson_ref ));
			else if ( lesson_type == LessonType.PARAGRAPH )
				this.append(
					new ParagraphLesson( series_id,resource,
								lesson_ref ));
			else // if ( lesson_type == LessonType.TUTORIAL )
				this.append(
					new TutorialLesson( series_id,resource,
								lesson_ref ));
		}
	}

	/**
	 * Constructor to create an empty lesson series.
	 */
	public LessonSeries( String series_id, String descr ) {
		this.series_id	= series_id;
		this.descr	= descr;
	}


	/*
	 * Accessor methods.
	 */
	public String	get_series_id()	{ return ( series_id ); }
	public String	get_descr()	{ return ( descr ); }
	public int	get_length()	{ return ( lessons.size() ); }


	/**
	 * Add a new lesson to this series.
	 */
	public void append( Lesson lesson ) {
		lessons.addElement( lesson );
	}

	/**
	 * Get a lesson back out of this series.
	 */
	public Lesson get( int index ) {
		return ( (Lesson)lessons.elementAt( index ) );
	}


	/**
	 * Printout method.
	 */
	public String toString() {
		String ret_str = getClass().getName()
				+ "{series_id=\"" + series_id
				+ "\",descr=\"" + descr
				+ "\",lessons@" + lessons.toString();
		return ( ret_str + "}" );
	}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		System.out.println( LessonSeries.create_lessonseries( "N" ) );
	}
}
